@@ -59,6 +59,10 @@ func TestOptionMarshalUnmarshal(t *testing.T) {
 			name: "recursive DNS servers",
 			subs: rdnssTests(),
 		},
+		{
+			name: "RA flags extension",
+			subs: raFlagsExtensionTests(),
+		},
 		{
 			name: "DNS search list",
 			subs: dnsslTests(),
@@ -67,6 +71,10 @@ func TestOptionMarshalUnmarshal(t *testing.T) {
 			name: "captive portal",
 			subs: cpTests(),
 		},
+		{
+			name: "pref64",
+			subs: pref64Tests(),
+		},
 		{
 			name: "nonce",
 			subs: nonceTests(),
@@ -100,7 +108,7 @@ func TestOptionMarshalUnmarshal(t *testing.T) {
 						t.Fatalf("failed to unmarshal options: %v", err)
 					}
 
-					if diff := cmp.Diff(st.os, got, cmp.Comparer(addrEqual)); diff != "" {
+					if diff := cmp.Diff(st.os, got, cmp.Comparer(addrEqual), cmp.Comparer(prefixEqual)); diff != "" {
 						t.Fatalf("unexpected options (-want +got):\n%s", diff)
 					}
 				})
@@ -276,6 +284,20 @@ func TestOptionUnmarshalError(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "ra flags extension",
+			o:    &RAFlagsExtension{},
+			subs: []sub{
+				{
+					name: "short flags",
+					bs: [][]byte{
+						{26, 1},
+						// Short flags.
+						ndptest.Zero(5),
+					},
+				},
+			},
+		},
 		{
 			name: "dnssl",
 			o:    &DNSSearchList{},
@@ -776,6 +798,70 @@ func rdnssTests() []optionSub {
 	}
 }
 
+func raFlagsExtensionTests() []optionSub {
+	return []optionSub{
+		{
+			name: "bad, no flags",
+			os: []Option{
+				&RAFlagsExtension{},
+			},
+		},
+		{
+			name: "bad, zero flags",
+			os: []Option{
+				&RAFlagsExtension{
+					Flags: RAFlags(ndptest.Zero(6)),
+				},
+			},
+		},
+		{
+			name: "bad, short padding",
+			os: []Option{
+				&RAFlagsExtension{
+					Flags: RAFlags{
+						0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
+						0x00,
+					},
+				},
+			},
+		},
+		{
+			name: "ok, length 1",
+			os: []Option{
+				&RAFlagsExtension{
+					Flags: RAFlags{0x80, 0x00, 0x00, 0x00, 0x00, 0x00},
+				},
+			},
+			bs: [][]byte{
+				{26, 1},
+				// Short values.
+				{128, 0, 0, 0, 0, 0},
+			},
+			ok: true,
+		},
+		{
+			name: "ok, length 2",
+			os: []Option{
+				&RAFlagsExtension{
+					Flags: RAFlags{
+						0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
+						0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+					},
+				},
+			},
+			bs: [][]byte{
+				{26, 2},
+				// Short values.
+				{
+					128, 0, 0, 0, 0, 0,
+					0, 0, 0, 0, 0, 0, 0, 0,
+				},
+			},
+			ok: true,
+		},
+	}
+}
+
 func dnsslTests() []optionSub {
 	return []optionSub{
 		{
@@ -939,6 +1025,101 @@ func cpTests() []optionSub {
 	}
 }
 
+func pref64Tests() []optionSub {
+	return []optionSub{
+		{
+			name: "bad, invalid prefix size",
+			os: []Option{
+				&PREF64{Prefix: netip.MustParsePrefix("2001:db8::/33"), Lifetime: time.Duration(0)},
+			},
+		},
+		{
+			name: "bad, invalid lifetime",
+			os: []Option{
+				&PREF64{Prefix: netip.MustParsePrefix("2001:db8::/32"), Lifetime: time.Hour * 24},
+			},
+		},
+		{
+			name: "ok, smallest prefix, max lifetime",
+			os: []Option{
+				&PREF64{Prefix: netip.MustParsePrefix("2001:db8::/96"), Lifetime: time.Second * 8 * 8191},
+			},
+			bs: [][]byte{
+				{0x26, 0x02}, {
+					0xff, 0xf8, 0x20, 0x01, 0x0d, 0xb8, 0x00,
+					0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+				},
+			},
+			ok: true,
+		},
+		{
+			name: "ok, /64 prefix",
+			os: []Option{
+				&PREF64{Prefix: netip.MustParsePrefix("2001:db8::/64"), Lifetime: time.Second * 8 * 8191},
+			},
+			bs: [][]byte{
+				{0x26, 0x02}, {
+					0xff, 0xf9, 0x20, 0x01, 0x0d, 0xb8, 0x00,
+					0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+				},
+			},
+			ok: true,
+		},
+		{
+			name: "ok, /56 prefix",
+			os: []Option{
+				&PREF64{Prefix: netip.MustParsePrefix("2001:db8::/56"), Lifetime: time.Second * 8 * 8191},
+			},
+			bs: [][]byte{
+				{0x26, 0x02}, {
+					0xff, 0xfa, 0x20, 0x01, 0x0d, 0xb8, 0x00,
+					0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+				},
+			},
+			ok: true,
+		},
+		{
+			name: "ok, /48 prefix",
+			os: []Option{
+				&PREF64{Prefix: netip.MustParsePrefix("2001:db8::/48"), Lifetime: time.Second * 8 * 8191},
+			},
+			bs: [][]byte{
+				{0x26, 0x02}, {
+					0xff, 0xfb, 0x20, 0x01, 0x0d, 0xb8, 0x00,
+					0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+				},
+			},
+			ok: true,
+		},
+		{
+			name: "ok, /40 prefix",
+			os: []Option{
+				&PREF64{Prefix: netip.MustParsePrefix("2001:db8::/40"), Lifetime: time.Second * 8 * 8191},
+			},
+			bs: [][]byte{
+				{0x26, 0x02}, {
+					0xff, 0xfc, 0x20, 0x01, 0x0d, 0xb8, 0x00,
+					0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+				},
+			},
+			ok: true,
+		},
+		{
+			name: "ok, maximum prefix, small lifetime",
+			os: []Option{
+				&PREF64{Prefix: netip.MustParsePrefix("2001:db8::/32"), Lifetime: time.Minute * 10},
+			},
+			bs: [][]byte{
+				{0x26, 0x02}, {
+					0x02, 0x5d, 0x20, 0x01, 0x0d, 0xb8, 0x00,
+					0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+				},
+			},
+			ok: true,
+		},
+	}
+}
+
 func nonceTests() []optionSub {
 	nonce := NewNonce()
 
